package com.hw.schedule;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hw.entity.GzPriceType;
import com.hw.entity.GzPrices;
import com.hw.entity.GzPricesLog;
import com.hw.service.GzPriceTypeService;
import com.hw.service.GzPricesLogService;
import com.hw.service.GzPricesService;
import com.hw.util.EasyHttpUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.*;

@Component
public class GzPricesSchedule {

    private Logger log = LoggerFactory.getLogger(GzPricesSchedule.class);

    @Autowired
    private GzPriceTypeService gzPriceTypeService;
    @Autowired
    private GzPricesService gzPricesService;
    @Autowired
    private GzPricesLogService gzPricesLogService;

    @Value("${open.url}")
    private String OPENURL;
    @Value("${open.authorization}")
    private String AUTHORIZATION;

    /**
     * 定时获取数据
     * lym
     */
    @Scheduled(cron = "0 0 2 2,5,12,20,25,30 * ?")
    public void schedulePullData(){
        pullData(AUTHORIZATION);
    }

    /**
     * 拉取数据
     * @param authorization
     */
    public String pullData(String authorization){
        log.info("开始获取上月分类价格数据"+ new Date());
        GzPriceType gzPriceType = new GzPriceType();
        List<GzPriceType> typeList = gzPriceTypeService.selectNotSynList(gzPriceType);
        int synNum = 0;
        if (typeList != null && typeList.size() > 0){
            for (GzPriceType priceType:typeList){
                String httpUrl = getUrl(priceType.getName()); //请求路径
                if (httpUrl != null || httpUrl != ""){ //拉取数据添加到数据库
                    synNum += getData(httpUrl,authorization,priceType.getId(),priceType.getName());
                }
            }
        }else {
            log.info("没有未同步上月的数据");
            return "没有未同步上月的数据";
        }
        log.info("结束获取上月分类价格数据,共添加" + synNum + "条数据。" + new Date() );
        if (synNum > 0){
            return "同步" + synNum + "条数据";
        }
        //0条可能数据平台还未出上月数据，也可能是访问数据平台401，500，503等
        return "同步" + synNum + "条数据,请查看原因!";
    }

    /**
     * 请求路径
     * lym
     * @param name
     * @return
     */
    private String getUrl(String name){
        String url = "";
        switch (name){
            case "水产类":
                url = "opene48cee28-96e5-4e2d-b702-70d8fb136e24?classifyId=E5747585BD137E5A5A4B94A943073CB7";
                break;
            case "蔬菜类":
                url = "open8d7af8cc-8992-4397-a57a-a65988e07a07?classifyId=CEEDE16B20FF344B8FC9F7FC9B590D74";
                break;
            case "辣椒类":
                url = "opena207271d-91c5-41d3-83d4-cf9b62942ddd?classifyId=704DA0BE1C7D3A31868E95AC58495AFD";
                break;
            case "水果类":
                url = "open2bb9b7b3-ef7b-424d-8078-6ce7b656e8cc?classifyId=D86F080303026D83CD7D08705F54FFDA";
                break;
            case "畜禽类":
                url = "opencb7131e5-a612-4bf5-b353-bccc7d02b237?classifyId=E5747585BD137E5A5A4B94A943073CB7";
                break;
            case "食用菌":
                url = "open6118232c-e421-403d-a18d-098b2b5936ec?classifyId=8ED038764DA0F2C86285EDB0426EF7B1";
                break;
            default:
                break;
        }

        String httpUrl = "";
        if (url != ""){
            httpUrl = OPENURL + url + "&dataFormat=json&interfaceId=getProductMarketPriceByMonth&dataCode=aec_price_info&limitCnt=1000&userToken=E4D380711ADD4962946C5EFE20ED382D";
        }
        return httpUrl;
    }

    /**
     * 查询开放
     * @param httpUrl
     */
    private int getData(String httpUrl,String authorization,int priceTypeId,String typeName){
        if (authorization == null || authorization == ""){
            authorization = AUTHORIZATION;
        }
        //header请求数据
        List<Map<String,String>> headerParams = new ArrayList<>();
        Map<String,String> params = new HashMap<>();
        params.put("name","Authorization");
        params.put("value",authorization);
        headerParams.add(params);
        //查询数据开放平台信息
        String data = EasyHttpUtils.LoadInstance().get(httpUrl,headerParams);
        int synNum = 0;
        log.info(data);
        if (data == null || data == "") return synNum;

        JSONObject jsonObject = JSONObject.parseObject(data);
        int rowCount = 0;
        if (jsonObject.containsKey("rowCount")){ //查询到的数据
             rowCount = (int) jsonObject.get("rowCount");
        }
        if (rowCount > 0 && jsonObject.containsKey("DS")){ //分解数据
            JSONArray list = (JSONArray) jsonObject.get("DS");
            List<GzPrices> prices = new ArrayList<>();
            String synTime = null; //拉取的数据时间
            for (Object object:list){
                JSONObject obj = (JSONObject) object;
                GzPrices gzPrices = new GzPrices();
                gzPrices.setMarket(obj.getString("市场"));
                gzPrices.setName(obj.getString("产品"));
                gzPrices.setUpdateTime(obj.getString("上传时间"));
                gzPrices.setPrice(obj.getString("价格"));
                gzPrices.setPriceUnit(obj.getString("价格单位"));
                gzPrices.setPriceType(obj.getString("价格类型"));
                gzPrices.setDataOrigion("贵州省政府数据开放平台");
                gzPrices.setFkPriceTypeId(priceTypeId);
                gzPrices.setLable(typeName);
                prices.add(gzPrices);
                if (synTime == null || synTime == ""){
                    synTime = obj.getString("上传时间");
                }
            }

            if (prices != null && prices.size() > 0){ //添加数据
                int i = gzPricesService.batchSave(prices);
                log.info("添加"+i+"条");
                GzPricesLog log = new GzPricesLog();
                log.setFkPriceTypeId(priceTypeId);
                log.setSynNum(i);
                log.setSynTime(synTime);
                gzPricesLogService.save(log);
                synNum += i;
            }
        }
        return synNum;
    }
}
